package com.persagy.apm.alarmservice.alarmdic.controller;

import com.persagy.apm.alarmservice.alarmcondition.model.dto.AddAlarmConditionDTO;
import com.persagy.apm.alarmservice.alarmdic.model.dto.QueryAlarmDicDTO;
import com.persagy.apm.alarmservice.alarmdic.service.AlarmDicServiceImpl;
import com.persagy.apm.common.enums.AlarmDicTypeEnum;
import com.persagy.apm.common.model.vo.EnumVO;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;

/**
 * @author lixing
 * @version V1.0 2021/9/13 9:52 上午
 **/
@Api(tags = "报警字典")
@Validated
@RestController
@RequestMapping("alarmDic")
public class AlarmDicController {
    @Autowired
    AlarmDicServiceImpl alarmDicService;

    @ApiOperation(value = "查询报警字典数据")
    @PostMapping("/query")
    public CommonResult<PageList<EnumVO>> queryAlarmDic(
            @Valid @RequestBody QueryAlarmDicDTO queryAlarmDicDTO) {
        String type = queryAlarmDicDTO.getType();
        List<EnumVO> resultList;
        // 如果未获取到需要查询的字典类型，返回空列表
        AlarmDicTypeEnum dicTypeEnum = AlarmDicTypeEnum.getByType(type);
        if (dicTypeEnum == null) {
            resultList = new ArrayList<>();
            return ResultHelper.multi(resultList);
        }

        switch (dicTypeEnum) {
            case ALARM_CATEGORY:
                resultList = alarmDicService.queryCategoryEnum();
                break;
            case ALARM_LEVEL:
                resultList = alarmDicService.queryLevelEnum();
                break;
            case ALARM_NATURE:
                resultList = alarmDicService.queryNatureEnum();
                break;
            case ALARM_STATE:
                resultList = alarmDicService.queryStateEnum();
                break;
            case ALARM_TREAT_MODE:
                resultList = alarmDicService.queryTreatModeEnum();
                break;
            case ALARM_TREAT_STATE:
                resultList = alarmDicService.queryTreatStateEnum();
                break;
            default:
                resultList = new ArrayList<>();
        }
        return ResultHelper.multi(resultList);
    }
}
