package com.persagy.apm.alarmservice.alarmrecord.enums;

import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.Setter;
import org.apache.commons.lang.StringUtils;

/**
 * 报警处理状态
 *
 * @author lixing
 * @version V1.0 2021/9/20 11:08 上午
 **/
@AllArgsConstructor
public enum EnumAlarmTreatState {
    /**
     * 报警处理状态
     */
    UN_HANDLE("1", "未处理"),
    HANDLING("2", "处理中"),
    DONE("3", "处理完成"),
    // 用于兼容子系统业务，后期会移除该状态
    IGNORE("4", "已忽略");

    @Setter
    @Getter
    private String type;

    @Setter
    @Getter
    private String desc;

    /**
     * 根据type获取到枚举对象
     *
     * @param type 枚举type值
     * @return 枚举对象
     * @author lixing
     * @version V1.0 2021/5/31 4:06 下午
     */
    public static EnumAlarmTreatState getByType(String type) {
        if (StringUtils.isBlank(type)) {
            return null;
        }
        EnumAlarmTreatState[] enums = EnumAlarmTreatState.values();
        for (EnumAlarmTreatState item : enums) {
            if (type.equals(item.getType())) {
                return item;
            }
        }
        return null;
    }
}
