package com.persagy.apm.alarmservice.calculatemethod.service.impl;

import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.alarmservice.calculatemethod.dao.CalculateMethodMapper;
import com.persagy.apm.alarmservice.calculatemethod.service.ICalculateMethodService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.stereotype.Service;
import org.apache.commons.lang.StringUtils;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.persagy.apm.alarmservice.calculatemethod.model.*;
import com.persagy.apm.alarmservice.calculatemethod.model.dto.*;
import java.util.List;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.google.common.base.CaseFormat;
import com.persagy.apm.common.model.dto.Sort;
import org.springframework.util.CollectionUtils;

/**
 * 计算方式(CalculateMethod) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:43:58
 */
@Service
public class CalculateMethodServiceImpl extends ServiceImpl<CalculateMethodMapper, CalculateMethod> 
    implements ICalculateMethodService {
    
    /**
    * 创建计算方式
    * @return 计算方式主键
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public String createCalculateMethod(AddCalculateMethodDTO addCalculateMethodDTO) {
        CalculateMethod calculateMethod = ConvertCalculateMethodTool.INSTANCE.convertAddDto2Entity(addCalculateMethodDTO);
        // 设置默认值
        setDefaultValue(calculateMethod);
        save(calculateMethod);
        return calculateMethod.getId();
    }
    
    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param calculateMethod 计算方式实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(CalculateMethod calculateMethod) {
        calculateMethod.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性
        
    }
    
    /**
    * 计算方式详情
    * @param id 主键
    * @return 部门do类
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public CalculateMethod queryCalculateMethodDetail(String id) {
        CalculateMethod calculateMethod = getById(id);
        if (calculateMethod == null) {
            throw new IllegalArgumentException("查看CalculateMethod详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return calculateMethod;
    }
    
    /**
    * 更新计算方式
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public void updateCalculateMethod(UpdateCalculateMethodDTO updateCalculateMethodDTO) {
        CalculateMethod calculateMethod = getById(updateCalculateMethodDTO.getId());
        calculateMethod = ConvertCalculateMethodTool.INSTANCE.convertUpdateDto2Entity(calculateMethod, updateCalculateMethodDTO);
        calculateMethod.setModifier(AppContext.getContext().getAccountId());
        updateById(calculateMethod);
    }
    
    /**
     * 校验计算方式是否可删除
     *
     * @param id 计算方式主键
     * @return 计算方式do类
     * @author lixing
     * @version V1.0 2021-09-08 22:43:58
     */
    public CalculateMethod checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除CalculateMethod时发生异常，主键为空");
        }

        CalculateMethod calculateMethod = getById(id);

        if (calculateMethod == null) {
            throw new IllegalArgumentException("删除CalculateMethod时发生异常，找不到要删除的数据，id:" + id);
        }

        return calculateMethod;
    }
    
    /**
    * 删除计算方式
    * @param id 主键
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public void deleteCalculateMethod(String id) {
        // 校验是否可删除
        CalculateMethod calculateMethod = checkDeletable(id);

        calculateMethod.setValid(ValidEnum.FALSE.getType());
        updateById(calculateMethod);
    }
    
    /**
    * 查询计算方式
    * @return List<CalculateMethod>
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public List<CalculateMethod> queryCalculateMethodList(QueryCalculateMethodDTO queryCalculateMethodDTO) {
        QueryWrapper<CalculateMethod> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(CalculateMethod.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, CalculateMethod.PROP_CREATIONTIME);
        
        if (queryCalculateMethodDTO != null) {
                                
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryCalculateMethodDTO.getName())) {
                queryWrapper.like(CalculateMethod.PROP_NAME, queryCalculateMethodDTO.getName());
            }
                                        
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryCalculateMethodDTO.getFormula())) {
                queryWrapper.like(CalculateMethod.PROP_FORMULA, queryCalculateMethodDTO.getFormula());
            }
                                        
            // todo 需判断使用like还是eq
            if (StringUtils.isNotEmpty(queryCalculateMethodDTO.getMonitorIndicatorId())) {
                queryWrapper.like(CalculateMethod.PROP_MONITOR_INDICATOR_ID, queryCalculateMethodDTO.getMonitorIndicatorId());
            }
                                            
            if (queryCalculateMethodDTO.getIsDefault() != null) {
                queryWrapper.eq(CalculateMethod.PROP_IS_DEFAULT, queryCalculateMethodDTO.getIsDefault());
            }
                                            
        }
        
        return list(queryWrapper);
    }
    
    /**
    * 分页查询计算方式
    * @return IPage<CalculateMethod>
    * @author lixing
    * @version V1.0 2021-09-08 22:43:58
    */
    @Override
    public IPage<CalculateMethod> pageQueryCalculateMethod(PageQueryCalculateMethodDTO pageQueryCalculateMethodDTO) {
        QueryWrapper<CalculateMethod> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(CalculateMethod.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<CalculateMethod> pageParam = new Page<>(pageQueryCalculateMethodDTO.getPage(), pageQueryCalculateMethodDTO.getSize(),true);
        // 排序信息
        if(CollectionUtils.isEmpty(pageQueryCalculateMethodDTO.getOrders())){
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, CalculateMethod.PROP_CREATIONTIME);
        }else {
            List<Sort> orders = pageQueryCalculateMethodDTO.getOrders();
            for(Sort sort: orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE,sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }
                
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryCalculateMethodDTO.getName())) {
            queryWrapper.like(CalculateMethod.PROP_NAME, pageQueryCalculateMethodDTO.getName());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryCalculateMethodDTO.getFormula())) {
            queryWrapper.like(CalculateMethod.PROP_FORMULA, pageQueryCalculateMethodDTO.getFormula());
        }
                            
        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryCalculateMethodDTO.getMonitorIndicatorId())) {
            queryWrapper.like(CalculateMethod.PROP_MONITOR_INDICATOR_ID, pageQueryCalculateMethodDTO.getMonitorIndicatorId());
        }
                            
        if (pageQueryCalculateMethodDTO.getIsDefault() != null) {
            queryWrapper.eq(CalculateMethod.PROP_IS_DEFAULT, pageQueryCalculateMethodDTO.getIsDefault());
        }
                                    
        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public String getMethodName(String calculateMethodId) {
        if (StringUtils.isBlank(calculateMethodId)) {
            return null;
        }
        CalculateMethod calculateMethod = queryCalculateMethodDetail(calculateMethodId);
        return calculateMethod.getName();
    }
}
