package com.persagy.apm.alarmservice.group.alarmrule.controller;

import com.persagy.apm.alarmservice.alarmcondition.service.IAlarmConditionService;
import com.persagy.apm.alarmservice.group.alarmrule.dao.GroupAlarmRuleMapper;
import com.persagy.apm.alarmservice.group.alarmrule.model.ConvertGroupAlarmRuleTool;
import com.persagy.apm.alarmservice.group.alarmrule.model.GroupAlarmRule;
import com.persagy.apm.alarmservice.group.alarmrule.model.dto.*;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.GroupAlarmRuleItemVO;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.GroupAlarmRuleListItemVO;
import com.persagy.apm.alarmservice.group.alarmrule.service.IGroupAlarmRuleService;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import com.persagy.apm.common.excel.ExcelUtils;
import com.persagy.apm.common.response.ApmResponseUpsertVO;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.energyalarmstarter.alarmdata.feign.DmpResult;
import com.persagy.apm.energyalarmstarter.alarmdata.model.dto.alarmconfig.QueryAlarmConfigDTO;
import com.persagy.apm.energyalarmstarter.alarmdata.model.vo.AlarmConfigItem;
import com.persagy.apm.energyalarmstarter.alarmdata.service.AlarmConfigServiceImpl;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * 集团报警规则控制层
 * @author lixing
 * @version V1.0 2021-09-08 22:35:45
 */
@Api(tags = "集团报警规则") 
@Validated
@RestController
@RequestMapping("groupAlarmRules")
public class GroupAlarmRuleController {
    @Autowired
    private IGroupAlarmRuleService groupAlarmRuleService;
    @Autowired
    private GroupAlarmRuleMapper groupAlarmRuleMapper;
    @Autowired
    private IAlarmConditionService alarmConditionService;
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;

	@ApiOperation(value = "创建集团报警规则")
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createGroupAlarmRule(
	        @Valid @RequestBody AddGroupAlarmRuleDTO addGroupAlarmRuleDTO) {
        String id = groupAlarmRuleService.createGroupAlarmRule(addGroupAlarmRuleDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "集团报警规则详情")
    @PostMapping("/get")
    @ApiIgnore
    public CommonResult<GroupAlarmRuleItemVO> queryGroupAlarmRuleDetail(
            @Valid @RequestBody QueryGroupAlarmRuleDetailDTO queryGroupAlarmRuleDetailDTO) {
        GroupAlarmRule groupAlarmRule = groupAlarmRuleService.queryGroupAlarmRuleDetail(queryGroupAlarmRuleDetailDTO.getId());
        GroupAlarmRuleItemVO groupAlarmRuleItemVO = ConvertGroupAlarmRuleTool.INSTANCE.convert2ResponseItemDTO(groupAlarmRule);
        return ResultHelper.single(groupAlarmRuleItemVO);
    }


    @ApiOperation(value = "更新集团报警规则")
    @PostMapping("/update")
    @ApiIgnore
    public CommonResult<ApmResponseUpsertVO> updateGroupAlarmRule(
            @Valid @RequestBody UpdateGroupAlarmRuleDTO updateGroupAlarmRuleDTO) {
        groupAlarmRuleService.updateGroupAlarmRule(updateGroupAlarmRuleDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateGroupAlarmRuleDTO.getId()));
    }


    @ApiOperation(value = "删除集团报警规则")
    @PostMapping("/delete")
    public CommonResult deleteGroupAlarmRule(
            @RequestBody @Valid DeleteGroupAlarmRuleDTO deleteGroupAlarmRuleDTO) {
        groupAlarmRuleService.deleteGroupAlarmRule(deleteGroupAlarmRuleDTO.getId());
        alarmConditionService.deleteAlarmCondition(deleteGroupAlarmRuleDTO.getAlarmConditionId());
        projectAlarmRuleObjRelService.deleteBatch();
        return ResultHelper.success();
    }


    @ApiOperation(value = "集团报警规则列表")
    @PostMapping("/query")
    public CommonResult<PageList<GroupAlarmRuleListItemVO>> queryGroupAlarmRule(
            @Valid @RequestBody PageQueryGroupAlarmRuleDTO pageQueryGroupAlarmRuleDTO) {
        int page = pageQueryGroupAlarmRuleDTO.getPage();
        int size = pageQueryGroupAlarmRuleDTO.getSize();
        pageQueryGroupAlarmRuleDTO.setStartIndex((page-1)*size);
        int count = groupAlarmRuleMapper.queryCount(pageQueryGroupAlarmRuleDTO);
        List<GroupAlarmRuleListItemVO> groupAlarmRuleList = groupAlarmRuleMapper.queryList(pageQueryGroupAlarmRuleDTO);
        return ResultHelper.multi(groupAlarmRuleList,count);
    }
    
//    @ApiOperation(value = "集团报警规则列表, 分页查询")
//    @PostMapping("/query")
//    public CommonResult<PageList<GroupAlarmRuleListItemVO>> queryGroupAlarmRule(
//            @Valid @RequestBody PageQueryGroupAlarmRuleDTO pageQueryGroupAlarmRuleDTO) {
//        IPage<GroupAlarmRule> pagedList = groupAlarmRuleService.pageQueryGroupAlarmRule(pageQueryGroupAlarmRuleDTO);
//        IPage<GroupAlarmRuleListItemVO> pagedResultList = pagedList.convert(item -> {
//            try {
//               return ConvertGroupAlarmRuleTool.INSTANCE.convert2ResponseListItemDTO(item);
//            } catch (Exception e) {
//                throw new IllegalArgumentException("分页查询，GroupAlarmRule实体类转换为列表对象发生异常", e);
//            }
//        });
//        return ResultHelper.multi(pagedResultList);
//    }

    @Autowired
    AlarmConfigServiceImpl alarmConfigService;

    @ApiOperation(value = "")
    @PostMapping("/test")
    public void test() throws Exception{
        QueryAlarmConfigDTO queryAlarmConfigDTO = new QueryAlarmConfigDTO();
        DmpResult<List<AlarmConfigItem>> query = alarmConfigService.query(queryAlarmConfigDTO);
        System.out.println(query);
    }

    @ApiOperation(value = "读取表格")
    @PostMapping("/readExcel")
    public void readExcel() throws Exception{
        Workbook workbook = ExcelUtils.getWorkbook("/项目清单.xlsx");
        Sheet sheet = workbook.getSheet("慧云3.0及3.1版本清单");
        List<Map<String, String>> list = new ArrayList();
        for (int i = 0; i < sheet.getLastRowNum(); i++) {

        }
    }

}

