package com.persagy.apm.alarmservice.group.management.controller;

import com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeDetailVO;
import com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeInfoVO;
import com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeVO;
import com.persagy.apm.alarmservice.group.management.service.GroupManagementService;
import com.persagy.apm.common.response.CommonResult;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.common.utils.ResultHelper;
import com.persagy.apm.model.dto.EquipIdDTO;
import com.persagy.apm.model.dto.EquipIdListDTO;
import com.persagy.apm.model.dto.GroupManagementDTO;
import com.persagy.apm.model.vo.GroupManagementVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.List;
import java.util.Objects;

import static com.persagy.apm.alarmservice.group.management.service.impl.GroupManagementServiceImpl.getGroupTree;

/**
 * @author 易涛
 * @version 1.0
 * @date 2021/9/7 18:09
 */
@Api(tags = "分组管理")
@RestController
@RequestMapping("group-management")
public class GroupManagementController {

    @Autowired
    private GroupManagementService managementService;

    @ApiOperation(value = "获取分组树")
    @PostMapping("/eq_groups/tree")
    public CommonResult<List<GroupTreeVO>> queryGroupTree(){
        return ResultHelper.single(getGroupTree());
    }

    @ApiOperation(value = "获取分组信息")
    @PostMapping("/eq_groups/get")
    public CommonResult<GroupTreeDetailVO> queryTreeById(@Valid @RequestBody EquipIdDTO equipIdDTO){
        return ResultHelper.single(managementService.getGroup(equipIdDTO.getId()));
    }
    
    @ApiOperation(value = "分组条件（设备类+查询条件）+项目 获取设备  加上分页和根据设备名称查询")
    @PostMapping("/equips/query")
    public CommonResult<PageList<GroupManagementVo>> equipsQuery(@Valid @RequestBody GroupManagementDTO managementDTO){
        PageList<GroupManagementVo> pageList = managementService.equipsQuery(managementDTO);
        return ResultHelper.multi(pageList.getRecords(),pageList.getTotal());
    }

    @ApiOperation(value = "获取设备名称")
    @PostMapping("/equips/get")
    public CommonResult<GroupManagementVo> equipsQuery(@Valid @RequestBody EquipIdDTO equipIdDTO) {
        return ResultHelper.single(managementService.equipsGet(equipIdDTO.getId()));
    }

    @ApiOperation(value = "获取分组名称")
    @PostMapping("/equipsGroup/query")
    public CommonResult<List<GroupTreeInfoVO>> equipsGroupQuery(@Valid @RequestBody EquipIdListDTO equipIdListDTO){
        return ResultHelper.single(managementService.equipsGroupQuery(equipIdListDTO.getEquipIdList())) ;
    }

    @Scheduled(cron = "0 0 * * * ?")
    public void batchDelete(){
       managementService.batchDelete();
    }
}
