package com.persagy.apm.alarmservice.group.management.service;

import com.persagy.apm.alarmservice.group.management.model.GroupManagement;
import com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeDetailVO;
import com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeInfoVO;
import com.persagy.apm.common.model.vo.SimpleObjVO;
import com.persagy.apm.common.response.PageList;
import com.persagy.apm.model.dto.GroupManagementDTO;
import com.persagy.apm.model.vo.GroupManagementVo;

import java.util.List;
import java.util.Map;

/**
 * @author 易涛
 * @version 1.0
 * @date 2021/9/8 14:17
 */
public interface GroupManagementService{
    /**
     * 分组条件（设备类+查询条件）+项目 获取设备
     * @param managementDTO 查询条件
     * @author 易涛
     * @date 2021/9/8 16:54
     * @return java.util.List<com.persagy.apm.model.vo.GroupManagementVo>
     */
    PageList<GroupManagementVo> equipsQuery(GroupManagementDTO managementDTO);
    
    /**
     *
     * @param id 设备id
     * @author 易涛
     * @date 2021/9/9 16:04 
     * @return com.persagy.apm.model.vo.GroupManagementVo
     */
    GroupManagementVo equipsGet(String id);

    /**
     * 查询设备
     *
     * @param ids 设备id列表
     * @return 设备id -> 设备信息 map
     * @author lixing
     * @version V1.0 2021/9/20 1:15 下午
     */
    Map<String, GroupManagement> queryEquipsByIds(List<String> ids);

    /**
     * 根据设备id查询分组名称
     * @param ids 设备id数组
     * @author 易涛
     * @date 2021/9/10 10:02
     * @return com.persagy.apm.alarmservice.group.management.model.vo.GroupTreeDetailVO
     */
    List<GroupTreeInfoVO> equipsGroupQuery(List<String> ids);

    /**
     * 根据设备id查询分组名称
     *
     * @param objIds 设备id列表
     * @return 设备id->分组名称映射
     * @author lixing
     * @version V1.0 2021/9/20 1:14 下午
     */
    Map<String, SimpleObjVO> queryObjGroup(List<String> objIds);

    /**
     * 定时任务删除valid为0的数据
     * @author 易涛
     * @date 2021/9/14 15:55
     * @return void
     */
    void batchDelete();

    /**
     * 查询分组下的设备
     * @param condition 分组查询条件
     * @param classCode 设备类
     * @return 分组下的设备
     */
    List<GroupManagementVo> getEquipList(String condition, String classCode);

    /**
     * 使用默认分页条件查询分组下的设备
     *
     * @param groupManagementDTO 查询条件
     * @return 分组下的设备
     */
    List<GroupManagementVo> getEquipList(GroupManagementDTO groupManagementDTO);

    /**
     * 获取分组信息
     *
     * @param id 分组id
     * @return 分组信息
     * @author lixing
     * @version V1.0 2021/9/18 7:03 下午
     */
    GroupTreeDetailVO getGroup(String id);

    /**
     * 查询所有叶子节点分组
     *
     * @return 所有可用分组的列表
     * @author lixing
     * @version V1.0 2021/9/19 1:02 下午
     */
    List<GroupTreeDetailVO> queryAllLeafGroups();
}
