package com.persagy.apm.alarmservice.monitorindicator.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.base.CaseFormat;
import com.persagy.apm.alarmservice.calculatemethod.model.CalculateMethod;
import com.persagy.apm.alarmservice.calculatemethod.model.dto.QueryCalculateMethodDTO;
import com.persagy.apm.alarmservice.calculatemethod.service.ICalculateMethodService;
import com.persagy.apm.alarmservice.monitorindicator.dao.MonitorIndicatorMapper;
import com.persagy.apm.alarmservice.monitorindicator.model.ConvertMonitorIndicatorTool;
import com.persagy.apm.alarmservice.monitorindicator.model.MonitorIndicator;
import com.persagy.apm.alarmservice.monitorindicator.model.dto.AddMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.monitorindicator.model.dto.PageQueryMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.monitorindicator.model.dto.QueryMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.monitorindicator.model.dto.UpdateMonitorIndicatorDTO;
import com.persagy.apm.alarmservice.monitorindicator.model.vo.MonitorIndicatorListItemVO;
import com.persagy.apm.alarmservice.monitorindicator.service.IMonitorIndicatorService;
import com.persagy.apm.common.constant.enums.ValidEnum;
import com.persagy.apm.common.context.AppContext;
import com.persagy.apm.common.model.dto.Sort;
import org.apache.commons.lang.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * 监测指标(MonitorIndicator) service层
 *
 * @author lixing
 * @version V1.0 2021-09-08 22:44:37
 */
@Service
public class MonitorIndicatorServiceImpl extends ServiceImpl<MonitorIndicatorMapper, MonitorIndicator>
        implements IMonitorIndicatorService {
    @Autowired
    ICalculateMethodService calculateMethodService;

    /**
     * 创建监测指标
     *
     * @return 监测指标主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public String createMonitorIndicator(AddMonitorIndicatorDTO addMonitorIndicatorDTO) {
        MonitorIndicator monitorIndicator = ConvertMonitorIndicatorTool.INSTANCE.convertAddDto2Entity(addMonitorIndicatorDTO);
        // 设置默认值
        setDefaultValue(monitorIndicator);
        save(monitorIndicator);
        return monitorIndicator.getId();
    }

    /**
     * 如果某些字段没有赋值，使用默认的值
     *
     * @param monitorIndicator 监测指标实体
     * @author lixing
     * @version V1.0 2021/3/12 12:29 下午
     */
    private void setDefaultValue(MonitorIndicator monitorIndicator) {
        monitorIndicator.setCreator(AppContext.getContext().getAccountId());
        // todo 其他默认的属性

    }

    /**
     * 监测指标详情
     *
     * @param id 主键
     * @return 部门do类
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public MonitorIndicator queryMonitorIndicatorDetail(String id) {
        MonitorIndicator monitorIndicator = getById(id);
        if (monitorIndicator == null) {
            throw new IllegalArgumentException("查看MonitorIndicator详情时发生异常，找不到要查看的记录，id=" + id);
        }
        return monitorIndicator;
    }

    /**
     * 更新监测指标
     *
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public void updateMonitorIndicator(UpdateMonitorIndicatorDTO updateMonitorIndicatorDTO) {
        MonitorIndicator monitorIndicator = getById(updateMonitorIndicatorDTO.getId());
        monitorIndicator = ConvertMonitorIndicatorTool.INSTANCE.convertUpdateDto2Entity(monitorIndicator, updateMonitorIndicatorDTO);
        monitorIndicator.setModifier(AppContext.getContext().getAccountId());
        updateById(monitorIndicator);
    }

    /**
     * 校验监测指标是否可删除
     *
     * @param id 监测指标主键
     * @return 监测指标do类
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    public MonitorIndicator checkDeletable(String id) {
        if (id == null) {
            throw new IllegalArgumentException("删除MonitorIndicator时发生异常，主键为空");
        }

        MonitorIndicator monitorIndicator = getById(id);

        if (monitorIndicator == null) {
            throw new IllegalArgumentException("删除MonitorIndicator时发生异常，找不到要删除的数据，id:" + id);
        }

        return monitorIndicator;
    }

    /**
     * 删除监测指标
     *
     * @param id 主键
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public void deleteMonitorIndicator(String id) {
        // 校验是否可删除
        MonitorIndicator monitorIndicator = checkDeletable(id);

        monitorIndicator.setValid(ValidEnum.FALSE.getType());
        updateById(monitorIndicator);
    }

    /**
     * 查询监测指标
     *
     * @return List<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public List<MonitorIndicator> queryMonitorIndicatorList(QueryMonitorIndicatorDTO queryMonitorIndicatorDTO) {
        QueryWrapper<MonitorIndicator> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(MonitorIndicator.PROP_VALID, ValidEnum.TRUE.getType());
        // 默认按创建时间倒序排序
        queryWrapper.orderBy(true, false, MonitorIndicator.PROP_CREATIONTIME);

        if (queryMonitorIndicatorDTO != null) {

            if (queryMonitorIndicatorDTO.getIds() != null) {
                queryWrapper.in(MonitorIndicator.PROP_ID, queryMonitorIndicatorDTO.getIds());
            }

            if (StringUtils.isNotEmpty(queryMonitorIndicatorDTO.getName())) {
                queryWrapper.like(MonitorIndicator.PROP_NAME, queryMonitorIndicatorDTO.getName());
            }

            if (StringUtils.isNotEmpty(queryMonitorIndicatorDTO.getClassCode())) {
                queryWrapper.eq(MonitorIndicator.PROP_CLASS_CODE, queryMonitorIndicatorDTO.getClassCode());
            }

        }

        return list(queryWrapper);
    }

    /**
     * 分页查询监测指标
     *
     * @return IPage<MonitorIndicator>
     * @author lixing
     * @version V1.0 2021-09-08 22:44:37
     */
    @Override
    public IPage<MonitorIndicator> pageQueryMonitorIndicator(PageQueryMonitorIndicatorDTO pageQueryMonitorIndicatorDTO) {
        QueryWrapper<MonitorIndicator> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq(MonitorIndicator.PROP_VALID, ValidEnum.TRUE.getType());
        // 这里认为pageQueryDTO是经过校验的，肯定包含分页信息
        IPage<MonitorIndicator> pageParam = new Page<>(pageQueryMonitorIndicatorDTO.getPage(), pageQueryMonitorIndicatorDTO.getSize(), true);
        // 排序信息
        if (CollectionUtils.isEmpty(pageQueryMonitorIndicatorDTO.getOrders())) {
            // 默认按创建时间倒序排序
            queryWrapper.orderBy(true, false, MonitorIndicator.PROP_CREATIONTIME);
        } else {
            List<Sort> orders = pageQueryMonitorIndicatorDTO.getOrders();
            for (Sort sort : orders) {
                // 将驼峰转换为下划线格式
                sort.setColumn(CaseFormat.LOWER_CAMEL.to(CaseFormat.LOWER_UNDERSCORE, sort.getColumn()));
                queryWrapper.orderBy(true, sort.isAsc(), sort.getColumn());
            }
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryMonitorIndicatorDTO.getName())) {
            queryWrapper.like(MonitorIndicator.PROP_NAME, pageQueryMonitorIndicatorDTO.getName());
        }

        // todo 需判断使用like还是eq
        if (StringUtils.isNotEmpty(pageQueryMonitorIndicatorDTO.getClassCode())) {
            queryWrapper.like(MonitorIndicator.PROP_CLASS_CODE, pageQueryMonitorIndicatorDTO.getClassCode());
        }

        return getBaseMapper().selectPage(pageParam, queryWrapper);
    }

    @Override
    public void setIndicatorMethodCount(List<MonitorIndicatorListItemVO> indicators) {
        indicators.forEach(indicator -> {
            QueryCalculateMethodDTO queryCalculateMethodDTO = new QueryCalculateMethodDTO();
            queryCalculateMethodDTO.setMonitorIndicatorId(indicator.getId());
            List<CalculateMethod> calculateMethods = calculateMethodService.
                    queryCalculateMethodList(queryCalculateMethodDTO);
            int methodCount = calculateMethods == null ? 0 : calculateMethods.size();
            indicator.setMethodCount(methodCount);
        });
    }
}
