package com.persagy.apm.alarmservice.project.alarmrule.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.persagy.apm.alarmservice.group.alarmrule.dao.GroupAlarmRuleMapper;
import com.persagy.apm.alarmservice.group.alarmrule.model.vo.GroupAlarmRuleListItemVO;
import com.persagy.apm.alarmservice.project.alarmrule.dao.ProjectAlarmRuleMapper;
import com.persagy.apm.alarmservice.project.alarmrule.service.IProjectAlarmRuleService;

import com.persagy.apm.alarmservice.project.alarmruleobjrel.model.ProjectAlarmRuleObjRel;
import com.persagy.apm.alarmservice.project.alarmruleobjrel.service.IProjectAlarmRuleObjRelService;
import org.springframework.beans.factory.annotation.Autowired;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import com.persagy.apm.alarmservice.project.alarmrule.model.*;
import com.persagy.apm.alarmservice.project.alarmrule.model.vo.*;
import com.persagy.apm.alarmservice.project.alarmrule.model.dto.*;
import com.persagy.apm.common.response.*;
import com.persagy.apm.common.utils.ResultHelper;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import springfox.documentation.annotations.ApiIgnore;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 项目报警规则控制层
 * @author lixing
 * @version V1.0 2021-09-08 22:46:16
 */
@Api(tags = "项目报警规则") 
@Validated
@RestController
@RequestMapping("projectAlarmRules")
public class ProjectAlarmRuleController {
    @Autowired
    private IProjectAlarmRuleService projectAlarmRuleService;
    @Autowired
    private IProjectAlarmRuleObjRelService projectAlarmRuleObjRelService;

	@ApiOperation(value = "创建项目报警规则")
	@PostMapping("/create")
	public CommonResult<ApmResponseUpsertVO> createProjectAlarmRule(
	        @Valid @RequestBody AddProjectAlarmRuleDTO addProjectAlarmRuleDTO) {
        String id = projectAlarmRuleService.createProjectAlarmRule(addProjectAlarmRuleDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(id));
	}
	
	@ApiOperation(value = "项目报警规则详情")
    @PostMapping("/get")
    public CommonResult<ProjectAlarmRuleItemVO> queryProjectAlarmRuleDetail(
            @Valid @RequestBody QueryProjectAlarmRuleDetailDTO queryProjectAlarmRuleDetailDTO) {
        ProjectAlarmRule projectAlarmRule = projectAlarmRuleService.queryProjectAlarmRuleDetail(queryProjectAlarmRuleDetailDTO.getId());
        ProjectAlarmRuleItemVO projectAlarmRuleItemVO = ConvertProjectAlarmRuleTool.INSTANCE.convert2ResponseItemDTO(projectAlarmRule);
        return ResultHelper.single(projectAlarmRuleItemVO);
    }


    @ApiOperation(value = "更新项目报警规则")
    @PostMapping("/update")
    public CommonResult<ApmResponseUpsertVO> updateProjectAlarmRule(
            @Valid @RequestBody UpdateProjectAlarmRuleDTO updateProjectAlarmRuleDTO) {
        projectAlarmRuleService.updateProjectAlarmRule(updateProjectAlarmRuleDTO);
        return ResultHelper.single(new ApmResponseUpsertVO(updateProjectAlarmRuleDTO.getId()));
    }


    @ApiOperation(value = "删除项目报警规则")
    @PostMapping("/delete")
    public CommonResult deleteProjectAlarmRule(
            @RequestBody @Valid DeleteProjectAlarmRuleDTO deleteProjectAlarmRuleDTO) {
        projectAlarmRuleService.deleteProjectAlarmRule(deleteProjectAlarmRuleDTO.getId());
        return ResultHelper.success();
    }


    @ApiOperation(value = "项目报警规则列表")
    @ApiIgnore
    @PostMapping("/query")
    public CommonResult<PageList<ProjectAlarmRuleListItemVO>> queryProjectAlarmRule(
            @Valid @RequestBody QueryProjectAlarmRuleDTO queryProjectAlarmRuleDTO) {
        List<ProjectAlarmRule> projectAlarmRuleList = projectAlarmRuleService.queryProjectAlarmRuleList(queryProjectAlarmRuleDTO);
        List<ProjectAlarmRuleListItemVO> resultList = ConvertProjectAlarmRuleTool.INSTANCE.convert2List(projectAlarmRuleList);
        return ResultHelper.multi(resultList);
    }
    
    @ApiOperation(value = "项目报警规则列表, 分页查询")
    @PostMapping("/queryPage")
    public CommonResult<PageList<ProjectAlarmRuleListItemVO>> queryProjectAlarmRule(
            @Valid @RequestBody PageQueryProjectAlarmRuleDTO pageQueryProjectAlarmRuleDTO) {
        IPage<ProjectAlarmRule> pagedList = projectAlarmRuleService.pageQueryProjectAlarmRule(pageQueryProjectAlarmRuleDTO);
        if(pagedList.getRecords().size()>0){
            List<String> projectIds = pagedList.getRecords().stream().map(ProjectAlarmRule::getProjectId).collect(Collectors.toList());
            List<ProjectAlarmRuleObjRel> relationList = projectAlarmRuleObjRelService.queryRelationListByProjectIds(projectIds);
            Map<String, ProjectAlarmRuleObjRel> relationMap = relationList.stream().collect(Collectors.toMap(ProjectAlarmRuleObjRel::getProjectId, rule -> rule));
            IPage<ProjectAlarmRuleListItemVO> pagedResultList = pagedList.convert(item -> {
                try {
                    return ConvertProjectAlarmRuleTool.INSTANCE.convert2ResponseListItemDTO(item);
                } catch (Exception e) {
                    throw new IllegalArgumentException("分页查询，ProjectAlarmRule实体类转换为列表对象发生异常", e);
                }
            });
            pagedResultList.getRecords().forEach(projectAlarmRuleListItemVO -> {
                String projectId = projectAlarmRuleListItemVO.getProjectId();
                if(relationMap.containsKey(projectId)){
                    projectAlarmRuleListItemVO.setRelListItemList((List<ProjectAlarmRuleObjRel>) relationMap.get(projectId));
                }
            });
            return ResultHelper.multi(pagedResultList);
        }

        return ResultHelper.multi(new ArrayList<>());
    }
}

